	ifndef _I2C_asm
		#define _I2C_asm
	; ==============================================================		
			list				p=16F819
#include		<P16F819.inc>
#include		"I2C-Servo.h"
#include		"I2C.h"
	; ==============================================================		
			udata_shr
ADDRTEMP		res				1			
ADDRPORT		res				1
	; -------------------------------------
			udata
I2C_TEMP		res				1
I2C_BUF1		res				1
I2C_BUF2		res				1
I2C_BUF3		res				1
	; ==============================================================		
			extern			FSR_TEMP
			extern			Servo0Preset, ServoStatus
			extern			ServosActive
			extern			SavePositions, SaveTrim
			extern			RestorePositions, RestoreTrim
			extern			SetPositionDefaults, SetTrimDefaults
			extern			WriteConfigByte, ReadConfigByte
			extern			WriteOnOffMask, ReadOnOffMask
			extern			TempServoIndex, TempServoCmd
			extern			TempServoValue
	; ==============================================================		
_I2C_asm	code

	; --------------------------------------------------------------
	; ***	I2C_Init
	; ***	Initializes the I2C Bus on the PIC
I2C_Init
			global			I2C_Init

			banksel			PORTB
			movf				PORTB, w			; read port as a whole
			movwf			ADDRPORT
			
			movlw			BASE_ADDR			; set base address
			movwf			ADDRTEMP
						
			btfsc			ADDRPORT, 0		; A0 set device address
			bsf				ADDRTEMP, 1
			btfsc			ADDRPORT, 2		; A1
			bsf				ADDRTEMP, 2
			btfsc			ADDRPORT, 3		; A2
			bsf				ADDRTEMP, 3
			
			banksel			SSPADD
			;movlw			B'00110010'		; I2C address 25
			movf				ADDRTEMP, w
			movwf			SSPADD
			
			banksel			SSPCON
			movlw			B'00110110'
			movwf			SSPCON			
			
			return

	; --------------------------------------------------------------
	; ***	I2C_Handler
	; ***	Processes commands sent via the I2C bus
I2C_Handler
			global			I2C_Handler

	; --- SPSTAT Register --------
	; 7 SMP - Slew Rate Control bit
	; 6 CKE - SMBus Select bit
	; 5 D/A - Data/Address bit
	; 4 P   - STOP bit
	; 3 S   - START bit
	; 2 R/W - Read/Write bit
	; 1 UA  - Update Address
	; 0 BF  - Buffer Full Status bit
	;
	; Mask  (00101101)

	; --------------------------------------------------------------
	; The I2C code below checks for 5 states:
	; --------------------------------------------------------------
	;
	; State 1: I2C write operation, last byte was an address byte.
	; 		SSPSTAT bits: D_A = 0, S = 1, R_W = 0, BF = 1   (00001001)
	;
	; State 2: I2C write operation, last byte was a data byte.
	; 		SSPSTAT bits: D_A = 1, S = 1, R_W = 0, BF = 1   (00101001)
	;
	; State 3: I2C read operation, last byte was an address byte.
	; 		SSPSTAT bits: D_A = 0, S = 1, R_W = 1, BF = 0   (00001100)
	;
	; State 4: I2C read operation, last byte was a data byte.
	; 		SSPSTAT bits: D_A = 1, S = 1, R_W = 1, BF = 0   (00101100)
	;
	; State 5: Slave I2C logic reset by NACK from master.
	; 		SSPSTAT bits: D_A = 1, S = 1, R_W = 0, BF = 0   (00101000)
	;
	; For convenience, WriteI2C and ReadI2C functions have been used.
	; --------------------------------------------------------------

			banksel			SSPSTAT
			movf				SSPSTAT, w
			banksel			I2C_TEMP
			andlw			B'00101101'
			movwf			I2C_TEMP

	; State 1: I2C write operation, last byte was an address byte.
	; 		SSPSTAT bits: D_A = 0, S = 1, R_W = 0, BF = 1   (00001001)
State1			
			movlw			B'00001001'
			xorwf			I2C_TEMP, w
			btfss			STATUS, Z
			goto				State2
		
			bcf				ServoStatus, Ignore_Rest
			bcf				ServoStatus, I2C_DATA_FLAG

			ReadI2C
			ReleaseClock
			return

	; State 2: I2C write operation, last byte was a data byte.
	; 		SSPSTAT bits: D_A = 1, S = 1, R_W = 0, BF = 1   (00101001)
State2		
			movlw			B'00101001'
			xorwf			I2C_TEMP, w
			btfss			STATUS, Z
			goto				State3

			btfsc			ServoStatus, Ignore_Rest
			goto				State_Ignore
			
			btfsc			ServoStatus, I2C_DATA_FLAG
			goto				State2_1
			
			ReadI2C
			banksel			TempServoIndex
			movwf			TempServoIndex
			
			movlw			B'11111000'		; extract servo command
			andwf			TempServoIndex, w
			movwf			TempServoCmd
			
			movlw			B'00000111'		; extract servo number
			andwf			TempServoIndex, f
			
			call				CheckCMD
			ReleaseClock
			return
State2_1
			ReadI2C
			Banksel			TempServoValue		; get command parameter value
			movwf			TempServoValue
			
			call				CheckCMD
			ReleaseClock
			return
			
	; State 3: I2C read operation, last byte was an address byte.
	; 		SSPSTAT bits: D_A = 0, S = 1, R_W = 1, BF = 0   (00001100)
State3
			movlw			B'00001100'
			xorwf			I2C_TEMP, w
			btfss			STATUS, Z
			goto				State4

			call				CheckCMD
			return
			
	; State 4: I2C read operation, last byte was a data byte.
	; 		SSPSTAT bits: D_A = 1, S = 1, R_W = 1, BF = 0   (00101100)
State4
			movlw			B'00101100'
			xorwf			I2C_TEMP, w
			btfss			STATUS, Z
			goto				State5

			call				CheckCMD
			return
			
	; State 5: Slave I2C logic reset by NACK from master.
	; 		SSPSTAT bits: D_A = 1, S = 1, R_W = 0, BF = 0   (00101000)
State5
			movlw			B'00101000'		; A NACK was received when transmitting
			xorwf			I2C_TEMP, w		; data back from the master. Slave logic
			btfss			STATUS, Z			; is reset in this case. R_W = 0, D_A = 1
			goto				I2CErr			; and BF = 0
			
			return							; If we arent in State5, then something is
											; wrong.
I2CErr 
	;banksel PORTB
	;bcf PORTB,2	; WCOL
	;bcf PORTB,3	; R_W
	;bcf PORTB,5	; S
	;bcf PORTB,6	; D_A
	;bcf PORTB,7	; I2C_ERR
I2C_ERR_0	
	;banksel SSPSTAT
	;btfss SSPSTAT, WCOL
	;goto I2C_ERR_1
	;banksel PORTB
	;bsf PORTB,2	
I2C_ERR_1	
	;banksel SSPSTAT
	;btfss SSPSTAT, R_W
	;goto I2C_ERR_2
	;banksel PORTB
	;bsf PORTB,3
I2C_ERR_2	
	;banksel SSPSTAT
	;btfss SSPSTAT, S
	;goto I2C_ERR_3
	;banksel PORTB
	;bsf PORTB,5
I2C_ERR_3
	;banksel SSPSTAT
	;btfss SSPSTAT, D_A
	;goto I2C_ERR_4
	;banksel PORTB
	;bsf PORTB,6
I2C_ERR_4
	;banksel PORTB	
	;bsf PORTB, 7
	bsf DEBUG_PIN
	
			banksel			SSPCON
			bcf				SSPCON, SSPOV		; clear overflow flag
State_Ignore			
			ReadI2C
			ReleaseClock
			return

WriteI2C
			banksel			SSPSTAT
			btfsc			SSPSTAT, BF		; Is the buffer full?
			goto				WriteI2C			; Yes, keep waiting.
DoI2CWrite
			banksel			SSPCON
			bcf				SSPCON, WCOL		; Clear the WCOL flag.
			movwf			SSPBUF			; Write the byte in WREG
			btfsc			SSPCON, WCOL		; Was there a write collision?
			goto				DoI2CWrite
			bsf				SSPCON, CKP		; Release the clock.
			return
			





	; --------------------------------------------------------------
	; ***	CheckCMD
	; ***	Check commands received via I2C
CheckCMD

	; --------------------------------------------------------------
	; ***	COMMAND 0x00 - Set Servo Position (3 bytes)
CMD_00
			movlw			I2C_CMD_SetServoPosition
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_01

			btfsc			ServoStatus, I2C_DATA_FLAG
			goto				CMD_00_1
			
			bsf				ServoStatus, I2C_DATA_FLAG
			return
CMD_00_1
			SaveFSR
			SetFSR			Servo0Preset

			bcf				STATUS, C
			rlf				TempServoIndex, w	; * 2			
			addlw			.1
			addwf			FSR, f
			movf				TempServoValue, w
			movwf			INDF

			bcf				ServoStatus, I2C_DATA_FLAG
			bsf				ServoStatus, Ignore_Rest
			RestoreFSR
			return
			
	; --------------------------------------------------------------
	; ***	COMMAND 0x01 - Set Servo trim (3 bytes)
CMD_01			
			movlw			I2C_CMD_SetServoTrim
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_02
			
			btfsc			ServoStatus, I2C_DATA_FLAG
			goto				CMD_01_1
			
			bsf				ServoStatus, I2C_DATA_FLAG
			return
CMD_01_1
			SaveFSR				
			SetFSR			Servo0Preset
			
			bcf				STATUS, C
			rlf				TempServoIndex, w	; * 2			
			addwf			FSR, f
			movf				TempServoValue, w
			movwf			INDF

			bcf				ServoStatus, I2C_DATA_FLAG
			bsf				ServoStatus, Ignore_Rest
			RestoreFSR
			return
			
	; --------------------------------------------------------------
	; ***	COMMAND 0x02 - write servo positions to EEPROM
CMD_02
			movlw			I2C_CMD_Positions2EEPROM
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_03

			call				SavePositions
			banksel			ServoStatus
			bsf				ServoStatus, Ignore_Rest
			return

	; --------------------------------------------------------------
	; ***	COMMAND 0x03 - write servo trim values to EEPROM
CMD_03
			movlw			I2C_CMD_Trims2EEPROM
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_04

			call				SaveTrim
			banksel			ServoStatus
			bsf				ServoStatus, Ignore_Rest
			return

	; --------------------------------------------------------------
	; ***	COMMAND 0x04 - reset trims and position to center
CMD_04
			movlw			I2C_CMD_InitAll
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_05

			call				SetPositionDefaults
			call				SetTrimDefaults			
			banksel			ServoStatus
			bsf				ServoStatus, Ignore_Rest
			return

	; --------------------------------------------------------------
	; ***	COMMAND 0x05 - reset trims and position from EEPROM
CMD_05
			movlw			I2C_CMD_LoadAllFromEEPROM
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_06

			call				RestoreTrim
			call				RestorePositions
			banksel			ServoStatus
			bsf				ServoStatus, Ignore_Rest
			return

	; --------------------------------------------------------------
	; ***	COMMAND 0x06 - read servo position
CMD_06
			movlw			I2C_CMD_ReadServoPosition
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_07

			banksel			SSPSTAT
			btfss			SSPSTAT, R_W
			goto				CMD_06_9

			SaveFSR
			SetFSR			Servo0Preset

			bcf				STATUS, C
			rlf				TempServoIndex, w	; * 2			
			addlw			.1
			addwf			FSR, f
			movf				INDF, w
			call				WriteI2C

			banksel			ServoStatus
			bsf				ServoStatus, Ignore_Rest
			RestoreFSR
CMD_06_9
			return

	; --------------------------------------------------------------
	; ***	COMMAND 0x07 - read servo trim
CMD_07
			movlw			I2C_CMD_ReadTrimPosition
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_08

			banksel			SSPSTAT
			btfss			SSPSTAT, R_W
			goto				CMD_07_9

			SaveFSR
			SetFSR			Servo0Preset
			
			bcf				STATUS, C
			rlf				TempServoIndex, w	; * 2			
			addwf			FSR, f
			movf				INDF, w
			call				WriteI2C

			banksel			ServoStatus
			bsf				ServoStatus, Ignore_Rest
			RestoreFSR
CMD_07_9			
			return

	; --------------------------------------------------------------
	; ***	COMMAND 0x08 - write configuration byte
CMD_08
			movlw			I2C_CMD_WriteConfigByte
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_09
			
			btfsc			ServoStatus, I2C_DATA_FLAG
			goto				CMD_08_1
			
			bsf				ServoStatus, I2C_DATA_FLAG
			return
CMD_08_1
			movf				TempServoValue, w
			call				WriteConfigByte
			
			bcf				ServoStatus, I2C_DATA_FLAG
			bsf				ServoStatus, Ignore_Rest
			return

	; --------------------------------------------------------------
	; ***	COMMAND 0x09 - read configuration byte
CMD_09
			movlw			I2C_CMD_ReadConfigByte
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_0A

			banksel			SSPSTAT
			btfss			SSPSTAT, R_W
			goto				CMD_09_9

			call				ReadConfigByte
			call				WriteI2C

			banksel			ServoStatus
			bsf				ServoStatus, Ignore_Rest
CMD_09_9
			return
			
	; --------------------------------------------------------------
	; ***	COMMAND 0x0A - Set all positions to center
CMD_0A
			movlw			I2C_CMD_AllPositions2Center
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_0B

			call				SetPositionDefaults			
			
			banksel			ServoStatus
			bsf				ServoStatus, Ignore_Rest
			return
			
	; --------------------------------------------------------------
	; ***	COMMAND 0x0B - Set all trims to center
CMD_0B
			movlw			I2C_CMD_AllTrims2Center
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_0C

			call				SetTrimDefaults
			
			banksel			ServoStatus
			bsf				ServoStatus, Ignore_Rest
			return
			
	; --------------------------------------------------------------
	; ***	COMMAND 0x0C - Read OnOff Mask
CMD_0C
			movlw			I2C_CMD_ReadOnOffMask
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_0D

			banksel			SSPSTAT
			btfss			SSPSTAT, R_W
			goto				CMD_0C_9

			call				ReadOnOffMask
			call				WriteI2C
			
			banksel			ServosActive
			movwf			ServosActive
			bsf				ServoStatus, Ignore_Rest
CMD_0C_9
			return
			
	; --------------------------------------------------------------
	; ***	COMMAND 0x0D - Write OnOff Mask
CMD_0D
			movlw			I2C_CMD_WriteOnOffMask
			subwf			TempServoCmd, w
			btfss			STATUS, Z
			goto				CMD_FF

			btfsc			ServoStatus, I2C_DATA_FLAG
			goto				CMD_0D_1
			
			bsf				ServoStatus, I2C_DATA_FLAG
			return
CMD_0D_1
			movf				TempServoValue, w
			movwf			ServosActive
			
			call				WriteOnOffMask
			
			banksel			ServoStatus
			bcf				ServoStatus, I2C_DATA_FLAG
			bsf				ServoStatus, Ignore_Rest
			return
			
	; --------------------------------------------------------------
	; ***	COMMAND 0xFF - unknown command
CMD_FF
			bcf				ServoStatus, I2C_DATA_FLAG
			bsf				ServoStatus, Ignore_Rest
			return
	; ==============================================================		
	endif
	
	end
